#' @title
#' Fill NA value
#' @description
#' Fill NA value with last value
#' @param x a vector of integer, numeric or character values
#' @param type specifies type of filling : 1 LOCF, 2 NOCB, 3 LOCF then NOCB
#' @details
#' \code{na_fill} takes a vector of integers, numerics or characters
#' and replace NA with the last non-NA value
#' @examples
#' > na_fill(c(NA, 1:4, NA, 1:5, NA, NA, 1:3))
#' [1] NA 1 2 3 4 4 1 2 3 4 5 5 5 1 2 3
#' > na_fill(c(NA, 1:4, NA, 1:5, NA, NA, 1:3), type=3)
#' [1] 1 1 2 3 4 4 1 2 3 4 5 5 5 1 2 3
#' @export

na_fill <- function(x, type = 1, inplace = F) {
  if (is.object(x)) {
    stop("x is not a primitive type", call. = FALSE)
  }
  if (is.integer(x))
    Cna_fill_int(x, type, inplace)
  else if (is.double(x))
    Cna_fill_num(x, type, inplace)
  else if (is.character(x))
    Cna_fill_char(x, type, inplace)
  else
    stop("Unimplemented type : ", typeof(x))
}
