#' @title
#' Fill NA value by group
#' @description
#' Fill NA value with last or next value by group
#' @param dt a data.table
#' @param var name of variable with integer, numeric or character values
#' @param by name(s) of variable(s) which determines groups
#' @param type specifies type of filling : 1 LOCF, 2 NOCB, 3 LOCF then NOCB
#' @examples
#' > na_fill_grp(dt, var="etat", by="id")
#' @export
na_fill_grp <- function(dt, var, by, type = 1L, inplace = F) {
  grp = data.table:::forderv(dt, by=by, retGrp = TRUE)
  lapply(var, function(x) na_fill_grp_var(dt, x, grp, type, inplace))
}

na_fill_grp_var <- function(dt, var, grp, type = 1L, inplace = F) {
  x = dt[[var]]
  {
    if (is.object(x)) {
      stop("x is not a primitive type", call. = FALSE)
    }
    if (is.integer(x))
      Cna_fill_int_grp(x, grp, type, inplace)
    else if (is.double(x))
      Cna_fill_num_grp(x, grp, type, inplace)
    else if (is.character(x))
      Cna_fill_char_grp(x, grp, type, inplace)
    else stop("Unimplemented type : ", typeof(x))
  }
}