#include <Rcpp.h>
using namespace Rcpp;

// [[Rcpp::export]]
IntegerVector Cna_fill_int_grp(IntegerVector x, IntegerVector rows, unsigned int opt = 1, bool inplace = false) {
  R_xlen_t n = x.size();
  R_xlen_t nrows = rows.size();

  if (nrows != 0 && n != rows.size())
    stop("x and rows must have the same length");
  if (!rows.hasAttribute("starts"))
    stop("rows must have 'starts' attribute");

  IntegerVector grps = rows.attr("starts");
  R_xlen_t ngrps = grps.size();

  IntegerVector ret = inplace ? x : clone(x);

  for(int g=0; g<ngrps; g++) {
    R_xlen_t f = grps[g] - 1; // start indice of group g (C indice = R indice - 1)
    R_xlen_t l = g == (ngrps - 1) ? n : grps[g + 1] - 1; // last indice (n if last group)

    if (opt & 0b01) {
      for(R_xlen_t i = f + 1; i < l; i++) {
        R_xlen_t r  = nrows == 0 ? i : rows[i] - 1;
        R_xlen_t r1 = nrows == 0 ? i - 1 : rows[i - 1] - 1;
        if(IntegerVector::is_na(ret[r]) && !IntegerVector::is_na(ret[r1])) {
          ret[r] = ret[r1];
        }
      }
    }

    if (opt & 0b10) {
      for(R_xlen_t i = l - 1; i > f; i--) {
        R_xlen_t r  = nrows == 0 ? i : rows[i] - 1;
        R_xlen_t r1 = nrows == 0 ? i - 1 : rows[i - 1] - 1;
        if(IntegerVector::is_na(ret[r1]) && !IntegerVector::is_na(ret[r])) {
          ret[r1] = ret[r];
        }
      }
    }

  }
  return ret;
}

// [[Rcpp::export]]
StringVector Cna_fill_char_grp(StringVector x, IntegerVector rows, unsigned int opt = 1, bool inplace = false) {
  R_xlen_t n = x.size();
  R_xlen_t nrows = rows.size();

  if (nrows != 0 && n != rows.size())
    stop("x and rows must have the same length");
  if (!rows.hasAttribute("starts"))
    stop("rows must have 'starts' attribute");

  IntegerVector grps = rows.attr("starts");
  R_xlen_t ngrps = grps.size();
  StringVector ret = inplace ? x : clone(x);

  for(int g=0; g<ngrps; g++) {
    R_xlen_t f = grps[g] - 1; // start indice of group g (C indice = R indice - 1)
    R_xlen_t l = g == (ngrps - 1) ? n : grps[g + 1] - 1; // last indice (n if last group)

    if (opt & 0b01) {
      for(R_xlen_t i = f + 1; i < l; i++) {
        R_xlen_t r  = nrows == 0 ? i : rows[i] - 1;
        R_xlen_t r1 = nrows == 0 ? i - 1 : rows[i - 1] - 1;
        if(StringVector::is_na(ret[r]) && !StringVector::is_na(ret[r1])) {
          ret[r] = ret[r1];
        }
      }
    }

    if (opt & 0b10) {
      for(R_xlen_t i = l - 1; i > f; i--) {
        R_xlen_t r  = nrows == 0 ? i : rows[i] - 1;
        R_xlen_t r1 = nrows == 0 ? i - 1 : rows[i - 1] - 1;
        if(StringVector::is_na(ret[r1]) && !StringVector::is_na(ret[r])) {
          ret[r1] = ret[r];
        }
      }
    }

  }
  return ret;
}


// [[Rcpp::export]]
NumericVector Cna_fill_num_grp(NumericVector x, IntegerVector rows, unsigned int opt = 1, bool inplace = false) {
  R_xlen_t n = x.size();
  R_xlen_t nrows = rows.size();

  if (nrows != 0 && n != rows.size())
    stop("x and rows must have the same length");
  if (!rows.hasAttribute("starts"))
    stop("rows must have 'starts' attribute");

  IntegerVector grps = rows.attr("starts");
  R_xlen_t ngrps = grps.size();
  NumericVector ret = inplace ? x : clone(x);

  for(int g=0; g<ngrps; g++) {
    R_xlen_t f = grps[g] - 1; // start indice of group g (C indice = R indice - 1)
    R_xlen_t l = g == (ngrps - 1) ? n : grps[g + 1] - 1; // last indice (n if last group)

    if (opt & 0b01) {
      for(R_xlen_t i = f + 1; i < l; i++) {
        R_xlen_t r  = nrows == 0 ? i : rows[i] - 1;
        R_xlen_t r1 = nrows == 0 ? i - 1 : rows[i - 1] - 1;
        if(NumericVector::is_na(ret[r]) && !NumericVector::is_na(ret[r1])) {
          ret[r] = ret[r1];
        }
      }
    }

    if (opt & 0b10) {
      for(R_xlen_t i = l - 1; i > f; i--) {
        R_xlen_t r  = nrows == 0 ? i : rows[i] - 1;
        R_xlen_t r1 = nrows == 0 ? i - 1 : rows[i - 1] - 1;
        if(NumericVector::is_na(ret[r1]) && !NumericVector::is_na(ret[r])) {
          ret[r1] = ret[r];
        }
      }
    }

  }
  return ret;
}
